@@ -24,7 +24,6 @@ raw data specified in version discovery responses.
 import copy
 import re
 
-from positional import positional
 import six
 from six.moves import urllib
 
@@ -57,7 +56,6 @@ def _int_or_latest(val):
     return LATEST if val == 'latest' or val == LATEST else int(val)
 
 
-@positional()
 def get_version_data(session, url, authenticated=None):
     """Retrieve raw version data from a url.
 
@@ -278,42 +276,34 @@ def version_to_string(version):
 def _version_between(min_version, max_version, candidate):
     """Determine whether a candidate version is within a specified range.
 
-    :param min_version: Normalized lower bound.  May be None.  May be
-                        (LATEST, LATEST).
-    :param max_version: Normalized upper bound.  May be None.  May be
-                        (LATEST, LATEST).
-    :param candidate: Normalized candidate version to test.  May not be None.
+    :param min_version: The minimum version that is acceptable.
+                        None/empty indicates no lower bound.
+    :param max_version: The maximum version that is acceptable.
+                        None/empty indicates no upper bound.
+    :param candidate: Candidate version to test.  May not be None/empty.
     :return: True if candidate is between min_version and max_version; False
              otherwise.
-    :raises ValueError: If candidate is None or the input is not properly
-                        normalized.
+    :raises ValueError: If candidate is None.
+    :raises TypeError: If any input cannot be normalized.
     """
-    def is_normalized(ver):
-        return normalize_version_number(ver) == ver
+    if not candidate:
+        raise ValueError("candidate is required.")
+    candidate = normalize_version_number(candidate)
 
-    # A version can't be between a range that doesn't exist
-    if not min_version and not max_version:
+    # Normalize up front to validate any malformed inputs
+    if min_version:
+        min_version = normalize_version_number(min_version)
+    if max_version:
+        max_version = normalize_version_number(max_version)
+
+    # If the candidate is less than the min_version, it's not a match.
+    # No min_version means no lower bound.
+    if min_version and candidate < min_version:
         return False
 
-    if candidate is None:
-        raise ValueError("candidate cannot be None.")
-
-    if min_version is not None and not is_normalized(min_version):
-        raise ValueError("min_version is not normalized.")
-    if max_version is not None and not is_normalized(max_version):
-        raise ValueError("max_version is not normalized.")
-    if not is_normalized(candidate):
-        raise ValueError("candidate is not normalized.")
-    # This is only possible if args weren't run through _normalize_version_args
-    if max_version is None and min_version is not None:
-        raise ValueError("Can't use None as an upper bound.")
-
-    # If the candidate is less than the min_version, it's
-    # not a match.  None works here.
-    if min_version is not None and candidate < min_version:
-        return False
-
-    if max_version is not None and candidate > max_version:
+    # If the candidate is higher than the max_version, it's not a match.
+    # No max_version means no upper bound.
+    if max_version and candidate > max_version:
         return False
 
     return True
@@ -372,8 +362,18 @@ def _combine_relative_url(discovery_url, version_url):
     # effect on absolute urls.
     url = urllib.parse.urljoin(discovery_url.rstrip('/') + '/', version_url)
 
-    # Parse and recombine the result to squish double //'s from the above
-    return urllib.parse.urlparse(url).geturl()
+    # Sadly version discovery documents are common with the scheme
+    # and netloc broken.
+    parsed_version_url = urllib.parse.urlparse(url)
+    parsed_discovery_url = urllib.parse.urlparse(discovery_url)
+
+    return urllib.parse.ParseResult(
+        parsed_discovery_url.scheme,
+        parsed_discovery_url.netloc,
+        parsed_version_url.path,
+        parsed_version_url.params,
+        parsed_version_url.query,
+        parsed_version_url.fragment).geturl()
 
 
 def _version_from_url(url):
@@ -395,7 +395,6 @@ class Discover(object):
     DEPRECATED_STATUSES = ('deprecated',)
     EXPERIMENTAL_STATUSES = ('experimental',)
 
-    @positional()
     def __init__(self, session, url, authenticated=None):
         self._url = url
         self._data = get_version_data(session, url,
@@ -441,7 +440,6 @@ class Discover(object):
 
         return versions
 
-    @positional()
     def version_data(self, reverse=False, **kwargs):
         """Get normalized version data.
 
@@ -627,7 +625,9 @@ class Discover(object):
                 return data
             if _latest_soft_match(min_version, data['version']):
                 return data
-            if _version_between(min_version, max_version, data['version']):
+            # Only validate version bounds if versions were specified
+            if min_version and max_version and _version_between(
+                    min_version, max_version, data['version']):
                 return data
 
         # If there is no version requested and we could not find a matching
@@ -673,7 +673,6 @@ class EndpointData(object):
     possibilities.
     """
 
-    @positional()
     def __init__(self,
                  catalog_url=None,
                  service_url=None,
@@ -735,11 +734,20 @@ class EndpointData(object):
         new_data._saved_project_id = self._saved_project_id
         return new_data
 
+    def __str__(self):
+        """Produce a string like EndpointData{key=val, ...}, for debugging."""
+        str_attrs = (
+            'api_version', 'catalog_url', 'endpoint_id', 'interface',
+            'major_version', 'max_microversion', 'min_microversion',
+            'next_min_version', 'not_before', 'raw_endpoint', 'region_name',
+            'service_id', 'service_name', 'service_type', 'service_url', 'url')
+        return "%s{%s}" % (self.__class__.__name__, ', '.join(
+            ["%s=%s" % (attr, getattr(self, attr)) for attr in str_attrs]))
+
     @property
     def url(self):
         return self.service_url or self.catalog_url
 
-    @positional(3)
     def get_current_versioned_data(self, session, allow=None, cache=None,
                                    project_id=None):
         """Run version discovery on the current endpoint.
@@ -774,7 +782,6 @@ class EndpointData(object):
             discover_versions=True,
             min_version=min_version, max_version=max_version)
 
-    @positional(3)
     def get_versioned_data(self, session, allow=None, cache=None,
                            allow_version_hack=True, project_id=None,
                            discover_versions=True,
@@ -913,7 +920,6 @@ class EndpointData(object):
                                        self._saved_project_id)
         self.service_url = url
 
-    @positional(1)
     def _run_discovery(self, session, cache, min_version, max_version,
                        project_id, allow_version_hack, discover_versions):
         tried = set()
@@ -983,7 +989,7 @@ class EndpointData(object):
         min_version and max_version are already normalized, so will either be
         None or a tuple.
         """
-        url = urllib.parse.urlparse(self.url)
+        url = urllib.parse.urlparse(self.url.rstrip('/'))
         url_parts = url.path.split('/')
 
         # First, check to see if the catalog url ends with a project id
@@ -1017,8 +1023,10 @@ class EndpointData(object):
         except TypeError:
             pass
         else:
-            is_between = _version_between(min_version, max_version,
-                                          url_version)
+            # `is_between` means version bounds were specified *and* the URL
+            # version is between them.
+            is_between = min_version and max_version and _version_between(
+                min_version, max_version, url_version)
             exact_match = (is_between and max_version and
                            max_version[0] == url_version[0])
             high_match = (is_between and max_version and
@@ -1103,7 +1111,6 @@ class EndpointData(object):
         return _VERSION_HACKS.get_discover_hack(self.service_type, self.url)
 
 
-@positional()
 def get_discovery(session, url, cache=None, authenticated=False):
     """Return the discovery object for a URL.
 
