@@ -12,8 +12,6 @@
 
 import uuid
 
-import six
-
 from keystoneauth1 import fixture
 from keystoneauth1.tests.unit import utils
 
@@ -287,7 +285,7 @@ class V3TokenTests(utils.TestCase):
             # the endpoint content below easier.
             self.assertTrue(endpoint.pop('id'))
 
-        for interface, url in six.iteritems(endpoints):
+        for interface, url in endpoints.items():
             endpoint = {'interface': interface, 'url': url,
                         'region': region, 'region_id': region}
             self.assertIn(endpoint, service['endpoints'])
